--our own boot.lua gives us the boot [address, path] as arguments...
local bootfs = ...
-- ...but [lua bios, openloader] don't, so fall back to the usual way
bootfs = bootfs or computer.getBootAddress()
local tmpfs = computer.tmpAddress()

--store all functions we need in locals so userspace mischief can't replace them
--prefix 'k' (for kernel) on any functions userspace will also have a version of to ensure kernelspace
--function usage is intentional (ie prevent accidentally using the kernel's `load` in a userspace `loadfile`)
local envBase = _G --used for creating process environments
local assert, checkArg, error, kload, ipairs, next, pairs, rawequal, rawset, select, setmetatable, tostring, type = assert, checkArg, error, load, ipairs, next, pairs, rawequal, rawset, select, setmetatable, tostring, type
local invoke, component_type = component.invoke, component.type
local kpullSignal, shutdown, uptime = computer.pullSignal, computer.shutdown, computer.uptime
local co_create, co_status, kresume, kyield = coroutine.create, coroutine.status, coroutine.resume, coroutine.yield
local huge, math_type, random = math.huge, math.type, math.random
local find, format, match, sub = string.find, string.format, string.match, string.sub
local concat, pack, unpack = table.concat, table.pack, table.unpack

local big = math.maxinteger or huge --some large amount to read at once in kreadfile


--completely disable the environment, ensuring the kernel only uses local variables, to prevent userspace
--mischief (somehow editing _G through a mirror) from affecting kernelspace
_ENV = setmetatable({}, {
	__index = function(self, key) return error(format('accessed global %q', key), 2) end,
	__newindex = function(self, key) return error(format('assigned global %q', key), 2) end,
})


local isFile --function(obj:any):boolean, boolean -- returns (is obj a file?), (is obj an open file?)

--[[function(index:number, val:any, types:string...):string -- asserts `types` contains the type of `val`. similar to `checkArg`.
	recognizes all possible returns from `type()` as well as ['integer', 'float', 'file']. unlike io.type(), 'file' ignores whether the file is open or closed.
	on success, returns the element of `types` that matched.
	on failure, throws an error originating from the caller's caller that mimicks built-in type errors. `index` should be the 1-based index of the argument that was the wrong type.
]]
local checkArgEx
do
	-- tail-recursive varargs part of checkArgEx
	local function check(val, t, want, ...)
		if want == t then
			return want
		elseif want == 'integer' or want == 'float' then
			if math_type(val) == want then
				return want
			end
		elseif want == 'file' then
			if isFile(val) then
				return want
			end
		elseif not want then
			return nil
		end
		return check(val, t, ...)
	end

	function checkArgEx(i, val, ...)
		local t = type(val)
		local ok = check(val, t, ...)
		if ok then
			return ok
		else
			error(format('bad argument #%d (%s expected, got %s)', i, concat({...}, ' or '), t), 3)
		end
	end
end

-- vi: set ts=2:
