--environment sandboxing uses mirrors: tables that provide a read-only "reflection" of what's on the other side
local mirror --function(target:table[, mirror:table]):table -- create a mirror (or make a table into one) that reflects `target`

do
	local mirrors = setmetatable({}, { __mode = 'k' }) --key is mirror given to userspace, value is what it mirrors

	local function mirrorNext(self, prev)
		local reflected = assert(mirrors[self], 'mirror reflects nothing')
		local key = next(reflected, prev)
		return key, self[key] --ensure all accesses go through the mirror so table values are mirrored too
	end

	local mirrormt = {
		__metatable = '<mirror>', --hide the metatable to prevent replacing or modifying it
		__index = function(self, key)
			local reflected = assert(mirrors[self], 'mirror reflects nothing')
			local val = reflected[key] --if mirrors[self] is somehow nil, erroring is fine
			if type(val) == 'table' then val = mirror(val) end --recursively mirror tables on first access
			if val ~= nil then rawset(self, key, val) end
			return val
		end,
		__pairs = function(self)
			--reuse a single function with the mirror as the control variable, instead of creating a new function with
			--upvalues every time, for vague feelings of less memory usage (TODO benchmark that)
			return mirrorNext, self
		end,
		--mirrors allow assignment but don't do anything with it
	}

	function mirror(target, mirror)
		mirror = mirror or {}
		mirrors[mirror] = target
		setmetatable(mirror, mirrormt)
		return mirror
	end
end

-- vi: set ts=2:
