--init is intended to be configured per-computer, so it lives in /etc/
local initCo, initInfo = createProcess('/etc/init.lua', nil) --no parent
if not initCo then
	error(initInfo, 0)
end
runqueue[initCo] = { initInfo.path, n = 1 }


local reschedule, dispatchSignal

--given the yield values of a coroutine (process or thread), reschedule it as necessary or mark the process as crashed.
local function postResume(co, ok, reason, ...)
	if co_status(co) == 'dead' then
		processes[co], schedDeadline[co], schedSignal[co] = nil
		--TODO bubble to parent process

		--tostring could run userspace code and even yield, but at this point we're crashing so it doesn't matter
		if not ok then error('process crashed: ' .. tostring(reason), 0) end
	elseif not reason then
		--userspace called coroutine.yield(); TODO figure out what that's supposed to do
		--for now... yield until another process resumes it?
		schedDeadline[co], schedSignal[co] = nil
	elseif rawequal(reason, 'signal') then
		local timeout = ...
		schedDeadline[co] = timeout and (uptime() + timeout)
		schedSignal[co] = true
	elseif rawequal(reason, 'deadline') then
		local timeout = ...
		schedDeadline[co] = uptime() + timeout
		schedSignal[co] = nil
	end
end

--logic after kpullSignal returns, in its own function to use varargs instead of table.pack
local function postPull(signal, ...)
	local args
	for proc, deadline in pairs(schedDeadline) do
		if uptime() >= deadline then
			args = args or { n = 1 } --resume with a single nil
			schedDeadline[proc] = nil
			runqueue[proc] = args
		end
	end

	--intentionally override {resumed by deadline} with {resumed by signal}
	if signal then
		args = pack(signal, ...)
		for proc in pairs(schedSignal) do
			schedSignal[proc] = nil
			runqueue[proc] = args
		end
	end
end


--goto instead of while-true for vague feeling of performance (TODO benchmark this)
::tickScheduler::

--[[clear the run queue
this is a delicate formulation. if proc creates new threads, those are added as new keys to runnable, which
makes next's behavior undefined*; removing proc before resuming causes the next loop iteration to crash with
"invalid key to 'next'". unconditionally removing proc after is also wrong, because proc may have rescheduled
itself immediately. instead we assign proc a value it should never have (false) before, test if it's still set
to that value after, and remove it if so.

the outer loop is for the occasional case that some of the child processes end up before their parent in next's
order and so wouldn't be resumed until after another signal was pulled otherwise.

*is this c-like ub that can cause memory corruption and segfaults? the manual doesn't say! (TODO make this not ub)
]]
while next(runqueue) do
	for proc, args in pairs(runqueue) do
		runqueue[proc] = false
		postResume(proc, kresume(proc, unpack(args, 1, args.n)))
		if not runqueue[proc] then
			runqueue[proc] = nil
		end
	end
end

local deadline = nil --absolute deadline to wait until for a signal; only set to nonnil if there is something waiting
for proc, pdeadline in pairs(schedDeadline) do
	if not deadline or pdeadline < deadline then
		deadline = pdeadline
	end
end

if next(schedSignal) then
	deadline = deadline or huge
end

if deadline then --at least one process/thread is waiting
	postPull(kpullSignal(deadline - uptime()))
	goto tickScheduler --then do it again
elseif next(runqueue) then --nothing is scheduled; is something still runnable somehow?
	goto tickScheduler --jump back to drain the runqueue again
elseif next(processes) then --nothing scheduled or runnable; are there any processes at all?
	error('all processes are idle', 0) --deadlock
else --no processes remaining, everything exited cleanly
	shutdown()
end

-- vi: set ts=2:
